Attribute VB_Name = "basPEFileInfo"
Option Explicit

' ---------
' Constants
' ---------
Public Const MAX_PATH = 260

' Constants for Image file
Public Const IMAGE_NUMBEROF_DIRECTORY_ENTRIES = 16
Public Const IMAGE_SIZEOF_SHORT_NAME = 8
Public Const IMAGE_DIRECTORY_ENTRY_EXPORT = 0          ' Export Directory
Public Const IMAGE_DIRECTORY_ENTRY_IMPORT = 1          ' Import Directory

Public Const IMAGE_DIRECTORY_ENTRY_RESOURCE = 2        ' Resource Directory
Public Const IMAGE_DIRECTORY_ENTRY_EXCEPTION = 3       ' Exception Directory
Public Const IMAGE_DIRECTORY_ENTRY_SECURITY = 4        ' Security Directory
Public Const IMAGE_DIRECTORY_ENTRY_BASERELOC = 5       ' Base Relocation Table
Public Const IMAGE_DIRECTORY_ENTRY_DEBUG = 6           ' Debug Directory
Public Const IMAGE_DIRECTORY_ENTRY_COPYRIGHT = 7       ' Description String
Public Const IMAGE_DIRECTORY_ENTRY_GLOBALPTR = 8       ' Machine Value (MIPS GP)
Public Const IMAGE_DIRECTORY_ENTRY_TLS = 9             ' TLS Directory
Public Const IMAGE_DIRECTORY_ENTRY_LOAD_CONFIG = 10    ' Load Configuration Directory
Public Const IMAGE_DIRECTORY_ENTRY_BOUND_IMPORT = 11   ' Bound Import Directory in headers
Public Const IMAGE_DIRECTORY_ENTRY_IAT = 12            ' Import Address Table

Public Const IMAGE_FILE_RELOCS_STRIPPED = &H1
Public Const IMAGE_FILE_EXECUTABLE_IMAGE = &H2
Public Const IMAGE_FILE_AGGRESSIVE_WS_TRIM = &H10
Public Const IMAGE_FILE_LARGE_ADDRESS_AWARE = &H20
Public Const IMAGE_FILE_BYTES_REVERSED_LO = &H80
Public Const IMAGE_FILE_BYTES_REVERSED_HI = &H8000
Public Const IMAGE_FILE_32BIT_MACHINE = &H100
Public Const IMAGE_FILE_DEBUG_STRIPPED = &H200
Public Const IMAGE_FILE_REMOVABLE_RUN_FROM_SWAP = &H400
Public Const IMAGE_FILE_SYSTEM = &H1000
Public Const IMAGE_FILE_DLL = &H2000
Public Const IMAGE_FILE_UP_SYSTEM_ONLY = &H4000
Public Const IMAGE_FILE_LINE_NUMS_STRIPPED = &H4
Public Const IMAGE_FILE_LOCAL_SYMS_STRIPPED = &H8
Public Const IMAGE_FILE_16BIT_MACHINE = &H40

' ----------------------
' Structures for PE File
' ----------------------

'-----------------------------------------------
' Data Directory entry
Public Type IMAGE_DATA_DIRECTORY  ' 8 bytes
   RVA  As Long
   size  As Long
End Type
'-----------------------------------------------
' Optional header (all three parts)
Public Type IMAGE_OPTIONAL_HEADER       ' 232 bytes
     'Standard fields.
   Magic  As Integer
   MajorLinkerVersion  As Byte
   MinorLinkerVersion  As Byte
   SizeOfCode  As Long
   SizeOfInitializedData  As Long
   SizeOfUninitializedData  As Long
   AddressOfEntryPoint  As Long
   BaseOfCode  As Long
   BaseOfData  As Long
     'NT additional fields.
   ImageBase  As Long
   SectionAlignment  As Long
   FileAlignment  As Long
   MajorOperatingSystemVersion  As Integer
   MinorOperatingSystemVersion  As Integer
   MajorImageVersion  As Integer
   MinorImageVersion  As Integer
   MajorSubsystemVersion  As Integer
   MinorSubsystemVersion  As Integer
   Win32VersionValue  As Long
   SizeOfImage  As Long
   SizeOfHeaders  As Long
   CheckSum  As Long
   Subsystem  As Integer
   DllCharacteristics  As Integer
   SizeOfStackReserve  As Long
   SizeOfStackCommit  As Long
   SizeOfHeapReserve  As Long
   SizeOfHeapCommit  As Long
   LoaderFlags  As Long
   NumberOfRvaAndSizes  As Long     '96
   ' Data directories
   DataDirectory(0 To IMAGE_NUMBEROF_DIRECTORY_ENTRIES) As IMAGE_DATA_DIRECTORY   ' 17*8 + 96 = 232
End Type

'-----------------------------------------------
' COFF File header
Public Type IMAGE_COFF_HEADER     ' 20 bytes
   Machine  As Integer
   NumberOfSections  As Integer
   TimeDateStamp  As Long
   PointerToSymbolTable  As Long
   NumberOfSymbols  As Long
   SizeOfOptionalHeader  As Integer
   Characteristics  As Integer
End Type

'-----------------------------------------------
' PE File header without MS-DOS stub
Public Type IMAGE_PE_FILE_HEADER   ' 256 bytes
   Signature  As Long                           ' 4 bytes -- PE signature
   FileHeader As IMAGE_COFF_HEADER            ' 20 bytes -- This is the COFF header
   OptionalHeader As IMAGE_OPTIONAL_HEADER    ' 232 bytes
End Type

'-----------------------------------------------
' Section header
Public Type IMAGE_SECTION_HEADER    '40 bytes
   ImageName(0 To IMAGE_SIZEOF_SHORT_NAME) As Byte       ' 8 bytes
   Misc As Long
   VirtualAddress As Long
   SizeofRawData As Long
   PointerToRawData As Long
   PointerToRelocations As Long
   PointerToLinenumbers As Long
   NumberOfRelocations As Integer
   NumberOfLinenumbers As Integer
   Characteristics As Long
End Type

'-----------------------------------------------
' Export Directory table
Public Type IMAGE_EXPORT_DIRECTORY_TABLE  ' 40 bytes
   Characteristics  As Long
   TimeDateStamp  As Long
   MajorVersion  As Integer
   MinorVersion  As Integer
   Name As Long
   Base As Long
   NumberOfFunctions  As Long
   NumberOfNames As Long                 ' We need this one
   pAddressOfFunctions  As Long
   ExportNamePointerTableRVA  As Long     ' We need this one
   pAddressOfNameOrdinals  As Long
End Type

'-----------------------------------------------
'-----------------------------------------------
' List entry -- Used by LOADED_IMAGE (for MapAndLoad)
Public Type LIST_ENTRY         ' 8 bytes
   FLink As Long
   Blink As Long
End Type

'-----------------------------------------------
' Loaded Image -- For use with MapAndLoad
Public Type LOADED_IMAGE       ' 48 bytes (46 bytes packed)
   ModuleName As Long
   hFile As Long
   MappedAddress As Long         ' Base address of mapped file
   pFileHeader As Long           ' Pointer to IMAGE_PE_FILE_HEADER
   pLastRvaSection As Long       ' Pointer to first COFF section header (section table)??
   NumberOfSections As Long
   pSections As Long             ' Pointer to first COFF section header (section table)??
   Characteristics As Long       ' Image characteristics value
   fSystemImage As Byte
   fDOSImage As Byte
   Links As LIST_ENTRY            ' 2 longs
   SizeOfImage As Long
End Type


' -----------------------------
' Function to use with PE files
' -----------------------------

'''''BOOL MapAndLoad(
'''''   IN LPSTR ImageName,
'''''   IN LPSTR DllPath,
'''''   OUT PLOADED_IMAGE LoadedImage,
'''''   IN BOOL DotDll,
'''''   IN BOOL ReadOnly);

Public Declare Function MapAndLoad Lib "Imagehlp.dll" ( _
   ByVal ImageName As String, _
   ByVal DLLPath As String, _
   LoadedImage As LOADED_IMAGE, _
   DotDLL As Long, _
   ReadOnly As Long) As Long

'''''BOOL UnMapAndLoad(
'''''  IN PLOADED_IMAGE LoadedImage
''''');

Public Declare Function UnMapAndLoad Lib "Imagehlp.dll" ( _
   LoadedImage As LOADED_IMAGE) As Long

'''''LPVOID ImageRvaToVa(
'''''  IN PIMAGE_PE_FILE_HEADER NtHeaders,
'''''  IN LPVOID Base,
'''''  IN DWORD Rva,
'''''  IN OUT PIMAGE_SECTION_HEADER *LastRvaSection
''''');
 
Public Declare Function ImageRvaToVa Lib "Imagehlp.dll" ( _
   ByVal NTHeaders As Long, _
   ByVal Base As Long, _
   ByVal RVA As Long, _
   ByVal LastRvaSection As Long) As Long


' --------------
' For API errors
' --------------
Public Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
Public Const FORMAT_MESSAGE_IGNORE_INSERTS = &H200

Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" ( _
  ByVal dwFlags As Long, lpSource As Any, ByVal dwMessageId As Long, _
   ByVal dwLanguageId As Long, ByVal lpBuffer As String, ByVal nSize As Long, Arguments As Long) As Long

' ----------
' CopyMemory
' ----------
Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" ( _
   lpvDest As Any, lpvSource As Any, ByVal cbCopy As Long)

' ----
' Misc
' ----
Declare Function GetSystemDirectory Lib "kernel32" Alias "GetSystemDirectoryA" ( _
   ByVal lpBuffer As String, _
   ByVal nSize As Long _
) As Long

Declare Sub VBGetTarget Lib "kernel32" Alias "RtlMoveMemory" ( _
   Target As Any, _
   ByVal lPointer As Long, _
   ByVal cbCopy As Long)

' ------
' Other API
' ------
Declare Function lstrlenA Lib "kernel32" (ByVal lpsz As Long) As Long

' ------------
' Version Info
' ------------
Public Const LANG_ENGLISH = &H409         ' include files say = &H09 ??
   
Public Type VS_FIXEDFILEINFO
   dwSignature As Long
   dwStrucVersion As Long
   dwFileVersionMS As Long
   dwFileVersionLS As Long
   dwProductVersionMS As Long
   dwProductVersionLS As Long
   dwFileFlagsMask As Long
   dwFileFlags As Long
   dwFileOS As Long
   dwFileType As Long
   dwFileSubtype As Long
   dwFileDateMS As Long
   dwFileDateLS As Long
End Type

Public Declare Function GetFileVersionInfoSize Lib "version.dll" Alias "GetFileVersionInfoSizeA" ( _
   ByVal lptstrFilename As String, _
   lpdwHandle As Long _
) As Long

Public Declare Function GetFileVersionInfo Lib "version.dll" Alias "GetFileVersionInfoA" ( _
   ByVal lptstrFilename As String, _
   ByVal dwHandle As Long, _
   ByVal dwLen As Long, _
   lpData As Byte _
) As Long

Public Declare Function VerLanguageName Lib "version.dll" Alias "VerLanguageNameA" ( _
   ByVal wLang As Long, _
   ByVal szLang As String, _
   ByVal nSize As Long _
) As Long

Public Declare Function VerQueryValue Lib "version.dll" Alias "VerQueryValueA" ( _
   pBlock As Byte, _
   ByVal lpSubBlock As String, _
   lplpBuffer As Long, _
   puLen As Long _
) As Long

Public Function GetAPIErrorText(ByVal lError As Long) As String
    
   Dim sOut As String
   Dim sMsg As String
   Dim lret As Long
   
   GetAPIErrorText = ""
   sMsg = String$(256, 0)
   
   lret = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM Or _
                       FORMAT_MESSAGE_IGNORE_INSERTS, _
                       0&, lError, 0&, sMsg, Len(sMsg), 0&)
   
   sOut = "Error: " & lError & "(&H" & Hex(lError) & "): "
   If lret <> 0 Then
      ' Check for ending vbcrlf
      sMsg = Trim0(sMsg)
      If Right$(sMsg, 2) = vbCrLf Then sMsg = Left$(sMsg, Len(sMsg) - 2)
      sOut = sOut & Trim0(sMsg)
   Else
      sOut = sOut & "<No such error>"
   End If
   
   GetAPIErrorText = sOut
      
End Function
Public Sub RaiseApiError(ByVal e As Long)
Err.Raise vbObjectError + 29000 + e, App.EXEName & ".Windows", GetAPIErrorText(e)
End Sub

Public Function Trim0(sName As String) As String

' Keep left portion of string sName up to first 0. Useful with Win API null terminated strings.

Dim x As Integer
x = InStr(sName, Chr$(0))
If x > 0 Then Trim0 = Left$(sName, x - 1) Else Trim0 = sName

End Function

Function GetPEChar(vValue As Variant) As String

' Function returns name of constant for given value.

Dim sName As String
sName = ""

If (vValue And &H1) Then sName = sName & "   RELOCS_STRIPPED" & vbCrLf
If (vValue And &H2) Then sName = sName & "   EXECUTABLE_IMAGE" & vbCrLf
If (vValue And &H10) Then sName = sName & "   AGGRESSIVE_WS_TRIM" & vbCrLf
If (vValue And &H20) Then sName = sName & "   LARGE_ADDRESS_AWARE" & vbCrLf
If (vValue And &H80) Then sName = sName & "   BYTES_REVERSED_LO" & vbCrLf
If (vValue And &H8000) Then sName = sName & "   BYTES_REVERSED_HI" & vbCrLf
If (vValue And &H100) Then sName = sName & "   32BIT_MACHINE" & vbCrLf
If (vValue And &H200) Then sName = sName & "   DEBUG_STRIPPED" & vbCrLf
If (vValue And &H400) Then sName = sName & "   REMOVABLE_RUN_FROM_SWAP" & vbCrLf
If (vValue And &H1000) Then sName = sName & "   SYSTEM_FILE" & vbCrLf

If (vValue And &H2000) Then sName = sName & "   DLL" & vbCrLf
If (vValue And &H4000) Then sName = sName & "   UP_SYSTEM_ONLY" & vbCrLf

If (vValue And &H4) Then sName = sName & "   LINE_NUMS_STRIPPED" & vbCrLf
If (vValue And &H8) Then sName = sName & "   LOCAL_SYMS_STRIPPED" & vbCrLf
If (vValue And &H40) Then sName = sName & "   16BIT_MACHINE" & vbCrLf

If Right$(sName, 2) = vbCrLf Then sName = Left$(sName, Len(sName) - 2)
GetPEChar = sName

End Function

Function LPSTRtoBSTR(ByVal lpsz As Long) As String

' Input: a valid LPSTR pointer lpsz
' Output: a sBSTR with the same character array

Dim cChars As Long

' Get number of characters in lpsz
cChars = lstrlenA(lpsz)

' Initialize string
LPSTRtoBSTR = String$(cChars, 0)

' Copy string
CopyMemory ByVal StrPtr(LPSTRtoBSTR), ByVal lpsz, cChars

' Convert to Unicode
LPSTRtoBSTR = Trim0(StrConv(LPSTRtoBSTR, vbUnicode))

End Function

